% close all
% clear all

function  [ISOres, ISOparestStats]= EstimateMutationalParameters(rep)

% EstimateMutationalParameters.m
%
% Inference of phenotypic parameters, such as 
% the basic viral growth rate,  
% fifty percent inhibitory NVP concentration IC_50
% probability and intensity of NRTI effect (p_NRTI & eta_NRTI)
% fold resistance FR(q) and 
% fitness f(q) of single mutations q, which occured during in vitro HIV passage experiments.
%
% Loads the data on observed mutations and applied drug regimes
% and estimates the resistance gain and fitness loss conferred by the mutations
% (see Barbara A. Rath, Kaveh Pouran Yousef, David K. Katzenstein, 
% Robert W. Shafer, Christof Schutte, Max von Kleist, Thomas C. Merigan (2013) 
% "HIV-1 Evolution in Response to Triple Reverse Transcriptase Inhibitor Induced Selective 
% Pressure in vitro", PlosOne)
%
% Input:  The file getparameters.m with adjusted program variables. The
%         function requires MATLAB's Optimization toolbox.
% 
% Returns: 1. 'ISOres': a 3D-cell array of length K, where the i-th entry contains 
%                      a (N_i x M_i)-matrix of residual errors between experimentally 
%                      measured passage times and the theoretical model. 
%                      K is the number of different isolate strains. 
%                      N_i is the number of all possible permutations of mutational 
%                      parameters which give rise to different models of viral evolution
%                      subject to drug application. M_i is the number of repeated
%                      independent estimations, using random initial parameters
%                      for numberical optimization. i is the index of the isolate 
%                      strains from the set [1...K].
%
%          2. 'ISOparestStats': a 4D-cell array of length K, where the i-th entry contains 
%                      a (L_i x N_i x M_i)-matrix of estimated mutational and drug effect parameters
%                      measured passage times and the theoretical model. 
%                      L_i is the total number of parameters across all
%                      isolates. The size variables K, N_i, M_i and i
%                      are as described above for 'ISOres'.
%
% Order of using the code:
%  1. Adjust programm variables in getparameters.m (optional)
%  2. Run EstimateMutationalParameters.m
%  3. Run EstimationStats.m
%
% Max v. Kleist and Kaveh Yousef
%
% Copyright (C) 2013, Free University Berlin
% Contact: yoocef@mi.fu-berlin.de, vkleist@zedat.fu-berlin.de
%
% FOR ACADEMIC USE this program is free software; you can redistribute 
% it and/or modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
% USA.

%General parameters and settings. Do not change.
%close all; 
global isoVec isolate nrReplicates OnlyImportantMutations allMutPars SearchAlgorithm
global NVP Resistance Fitness BasicPars MaxNumberParameters
global NrPassages NrExperiments parIndex meanData stddevData  nrExp simultanExperiments

%parsing the input
if (ischar(rep)) 
    rep = str2num(rep);
end

%initialize program variables
getparameters();
 
options = optimset('Display','off');

% The mumber of parameters equals the number of all resistance parameters FR(q), 
% fitness parameters f(q)(see getparameters.m) plus the four baseline
% parameters r_0, IC50, p_NRTI, eta_NRTI;
nrAllPars=length(allMutPars)+4;

%--------------
% these are the number of genetically different baseline isolates, i.e.
% (the number that are treated differently during parameter estimation)
% In our case there were 5 baseline isolates, but only 4 genetically
% different ones, as isolate 2 & 3 were same. Hence, we parameters for
% Iso2/3 w�ll be estimated together
NrIsolates = length(isoVec); 

%Initialize cell-Arrays of Matrices
ISOparestStats = cell(1,NrIsolates);
ISOres = cell(1,NrIsolates);
ISOaic = cell(1,NrIsolates);
ISObic = cell(1,NrIsolates);
if OnlyImportantMutations
    ISOparperm = cell(NrIsolates,MaxNumberParameters(2)); 
else
    ISOparperm = cell(NrIsolates,MaxNumberParameters(1)); 
end
%---------------------

for k = 1:NrIsolates
  isolate = isoVec{k};%assign current isolate(s) for which parameters ought to be estimated
  [NVP,Resistance,Fitness,BasicPars] = getparameters(isolate);
    
  imprtnt=Resistance.important;
  if OnlyImportantMutations
    lastidx = imprtnt;
  else
    lastidx = length(Resistance.parIndex);
  end
  
  pars = [Fitness.parIndex, Resistance.parIndex(1:lastidx)];
  %addPars = [25, 26, 27];  %parameters r_0, epsMean1, prob1
  nrPerms = 0;
  permidx=0;
  
  NrOfParametersForIsolate = length(pars);

  for q = 1:length(pars)
    permutation = nchoosek(pars,q);%number of model permutations with 'q' out of 'pars' parameters
    sz=size(permutation,1);
    ISOparperm{k,q} = permutation;   
    nrPerms = nrPerms + sz;
    for qq=1:size(permutation,1)
      ISOparperm2.(strcat(isoVec{k}{:})){nrPerms-sz+qq} = permutation(qq,:);
    end
  end
  
  ISOparestStats{k} = nan(nrAllPars, nrPerms,nrReplicates);
  ISOres{k}=nan(nrPerms,nrReplicates);
  ISOaic{k}=nan(nrPerms,nrReplicates);
  ISObic{k}=nan(nrPerms,nrReplicates);
end
%----------------
%Vectors of boundaries (min/max) as constraints for parameter estimation.
%The i-th entry of the the two following vectors determines 
%the min (left = lower boundary) and max (right = upper boundary) constraints for i-th parameter.

leftBoundGeneral = zeros(1,nrAllPars); % all positive values

% FR(q) = 1/(1-estimate(q)); estimate is in the range in [0 1] ;
% f(q) = 1-estimate(q); estimate of fitness deficit is in the range in [0 1]
% r_0 (basic growth rate) epsMean1 (mean of stochastic NRTI effect) prob1 (probability of stochastic NRTI effect) IC50 value for dose-dependance of NVP drug effect
rightBoundGeneral = [ones(1,nrAllPars-4), inf, 1, 1, inf];
%clear ISOaic ISOparperm ISOparest ISOmeanData ISOstdData
%clear ISOres ISOresIdx ISOparestStats ISOmeanEst ISOstdEst 

format short;
fprintf('%s\n\n','Welcome to estimating HIV mutation parameters.');
fprintf('%s%u%s\n','Optimization will be conducted for ',NrIsolates,'genetically distinct basel. isolates.');
%%
%iterate over isolates
for j =1:NrIsolates 
  isolate = isoVec{j};
  fprintf('%s%s\n','Current basel. isolate: ',strcat(isoVec{j}{:}))
  fprintf('%s%u%s\n','Nr. of models (param. permutations) to scan for this isolate: ',size(ISOres{j},1),'         ')
  %import muational events q in Q(j,p) for isolate and their respective
  %indicator functions
  [NVP,Resistance,Fitness, BasicPars,passageData] = getparameters(isolate);
  imprtnt=Resistance.important;
  if OnlyImportantMutations
    lastidx = imprtnt;
  else
    lastidx = length(Resistance.parIndex);
  end
  pars = [Fitness.parIndex, Resistance.parIndex(1:lastidx)];
  addPars = nrAllPars-3:nrAllPars;
  nrPars=length(pars);
  [NrExperiments,NrPassages] = size(NVP);
  % the first collumn indicates the passage number, while collumn 2 to 7
  % indicate the passage times = > mean over collumns 2:7
  meanData = nanmean(passageData(:,2:end)); 
  stddevData = nanstd(passageData(:,2:end));
  % For the weigthed least square, this denotes the terms mu_exp/mu_exp and
  % sigma_exp/sigma_exp; hence 1
  ydata  = ones(1,nrExp.default*2+1)';  
  currPerm=1;
  permcnt=0;% model (parameter permutation) counter
  fprintf('%s','Model Nr# ')
  reverseStr='';
  
  %iterate over parameter permutations
  for i =1:nrPars 
    nrPerms = size(ISOparperm{j,i},1);        
    % iterate over all possible permutations with q mutational parameters;
    % i.e. all models with 4 mutational parameters
    for q = 1:nrPerms; 
      permcnt=permcnt+1;
      fprintf(reverseStr);
      fprintf('%u\n',permcnt)
      
      parIndex= [ISOparperm{j,i}(q,:), addPars(2:end)];
      
      %--------------------------------------------------------------------
      % simultaneous minimization of FR parameters
      NumberResistanceParametersInPermut = sum(ISOparperm{j,i}(q,:)>= 7);
      if NumberResistanceParametersInPermut
        ydata_new = [ydata;zeros(NumberResistanceParametersInPermut,1)];
      else
         ydata_new = ydata;
      end
      %--------------------------------------------------------------------
      %
      leftBound = leftBoundGeneral(parIndex);
      rightBound = rightBoundGeneral(parIndex);
      for currRep=1:nrReplicates % iterate over all replicates of the parameter estimation for one specific model
        fprintf('%s','.')
        initGeneral =rand(1,nrAllPars);%generate random start parameters for optimization
        init = initGeneral(parIndex);
        success=0;
        switch SearchAlgorithm
          case 1
            if length(parIndex) > 2*length(simultanExperiments)+1 % unconstrained optimization (Levenberg-Marquard) in the case where the number of parameters exceeds the number of data points 
              for ii=1:3
                init = initGeneral(parIndex);
                %This part corresponds to eq. (11) in the manuscript 
                [k_fit,resnorm] = lsqcurvefit(@FitStoch,init,simultanExperiments,ydata_new,[],[],options);
                if isreal(k_fit) && sum(k_fit>=0)==length(parIndex)
                  [stats, meanResult, stdResult] = FitStoch(k_fit, simultanExperiments);
                  success=1;
                  break;
                end
              end
            else  % constrained optimization (trust-region-reflective alg.) in the case where the number of parameters is smaller than the number of data points 
              %This part corresponds to eq. (11) in the manuscript 
              [k_fit,resnorm] = lsqcurvefit(@FitStoch,init,simultanExperiments,ydata_new,leftBound,rightBound,options); 
              success=1;
            end
          case 2 % unconstrained optimization by simplex method in the case where this algorithm was pre-defined
            %This part corresponds to eq. (11) in the manuscript 
            [k_fit, resnorm] = fminsearch(@(x) FitStoch(x,simultanExperiments), init, options);
            success=1;
        end
        if success
          %store parameter estimates and goodness of fit 
          AIC= 12*log(resnorm) + 2*length(parIndex);
          BIC = 12*log(resnorm) + log(12)*length(parIndex);
          ISOaic{j}(currPerm,currRep)=AIC;
          ISObic{j}(currPerm,currRep)=BIC;
          ISOres{j}(currPerm, currRep)=resnorm;
          ISOparestStats{j}(parIndex,currPerm, currRep) = k_fit;
        end
      end
      currPerm=currPerm+1;
      %reverseStr = repmat(sprintf('\b'), 1, nrReplicates+length(num2str(permcnt))+1);
    end
  end
end

%Uncomment if results should be saved to a .mat file
clearvars -except v1 v2 isoVec ISOres ISOparestStats ISOaic ISOparperm2 ISObic rep
save (strcat('EstimationResults_repeat_',num2str(rep),'.mat'))

