% EstimationStats.m
%
% Inference of resistance and fitness values using HIV passage experiment data.
%
% This program conducts and visualizes statistics on estimation data (see Barbara A. Rath, 
% Kaveh Pouran Yousef, David K. Katzenstein, Robert W. Shafer, Christof Schütte, 
% Max von Kleist, Thomas C. Merigan (2013) "HIV-1 Evolution in Response to Triple Reverse 
% Transcriptase Inhibitor Induced Selective 
% Pressure in vitro", PlosOne)
%
% The program loads a .mat file specified further below and visualizes the
% statistical properties of corresponding estimation results and/or writes
% estimation statistics to an output file. In order to
% determine which plots or output files need to be generated,
% the corresponding MACROS need to be adjusted (see below).
%
% Order of using the code:
%  1. Adjust program variables in getparameters.m (optional)
%  2. Run EstimateMutationalParameters.m
%  3. Run EstimationStats.m
%
% Max v. Kleist and Kaveh Yousef, January 2013
%
% Copyright (C) 2013, Free University Berlin
% Contact: yoocef@mi.fu-berlin.de, vkleist@zedat.fu-berlin.de
%
% FOR ACADEMIC USE this program is free software; you can redistribute 
% it and/or modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
% USA.

getparameters();
global allMutPars nrFitnessMuts nrResistanceMuts isoVec nrExp simultanExperiments parIndex
global NVP Resistance Fitness meanData nrExp  stddevData BasicPars isolate
close all

%------------------------------------------------------
%Adjust the name of the .mat-file to be evaluated
load 'EstimationResultsALL.mat'
%load 'EstimationResults_Iso2_3_newobjfct_complete.mat'
%------------------------------------------------------

%-------------------------
%Adjust following MACROS (0/1):

%Residual curve
SHOW_AIC_CURVE=0;

%Print parameters to file
PRINT_TO_FILE=1;

%A file name string 
OUTFILENAME = 'parestStats.out';

%Generate boxplots of the distribution of each parameter
%within the best models
PAR_BOXPLOTS=1;

%Plot predicted vs. measured passage times
PRED_CHECKS=1;

%
RellikelihoodThershold = 0.45; %cut-off value for model selection
%--------------------------


nrIsolates=length(isoVec);
figcnt = 0;

FCnames=cell(1,nrFitnessMuts);
FRnames=cell(1,nrResistanceMuts);
lenmutpars=length(allMutPars);
for i=1:lenmutpars
  currmut = allMutPars{i};
  idx = strfind(currmut,'_');
  if i <= nrFitnessMuts
    if length(idx)
      FCnames{i}= strcat(currmut(1:idx-1),'\_',currmut(idx+1:end));
    else
      FCnames{i}=currmut;
    end
  else
    if length(idx)
      FRnames{i-nrFitnessMuts}= strcat(currmut(1:idx-1),'\_',currmut(idx+1:end));
    else
      FRnames{i-nrFitnessMuts}=currmut;
    end
  end
end
FCidx=1:nrFitnessMuts;
FRidx=nrFitnessMuts+1:lenmutpars;
BaseParNames={'k','epsilon1','prob1','IC50'};
BaseParidx=[lenmutpars+1:lenmutpars+4];
allParNames=[allMutPars, BaseParNames];
nrAllPars=length(allParNames);
%Compute relative cut-off for the best models
%using relative likelihood basid on AIC
ISOrelLikelihood = cell(nrIsolates,1);
IDX = cell(nrIsolates,1);
for j=1:nrIsolates%length(isoVec)
  meanVec=mean(ISOaic{j},2);
  [Y,I]=sort(meanVec, 'ascend');
  IDX{j} = I;
  ISOaic{j} = ISOaic{j}(I,:);
  ISOparestStats{j}=ISOparestStats{j}(:,I,:);
  ISOparperm2.(strcat(isoVec{j}{:})) = ISOparperm2.(strcat(isoVec{j}{:}))(I)
  [n,m] = size(ISOres{j});
  Iso_X_Rellikelihood_ = zeros(1,n);
  ISO_X_AICmin = median(ISOaic{j}(1,:));
  for jj = 1:n
    Iso_X_Rellikelihood_(jj) = exp((ISO_X_AICmin-median(ISOaic{j}(jj,:)))/2);
    if Iso_X_Rellikelihood_(jj) < RellikelihoodThershold
      Iso_X_Rellikelihood = Iso_X_Rellikelihood_(1:jj-1);
      cutVec(j) = jj-1;
      break;
    end
  end
  ISOrelLikelihood{j} = Iso_X_Rellikelihood;
end

medianCutoff=cell(nrIsolates);
bestParsCutVals=cell(nrIsolates,nrAllPars);

%Print out the parameters for the best models to the GUI
for j=1:length(isoVec)
  medianCutoff{j}=zeros(3,nrAllPars);
  fprintf('%s\n','--------------------------------------------------');
  fprintf('%s%s\n','Best models for isolate ',strcat(isoVec{j}{:}));
  for jj = 1:cutVec(j)
    %idx=IDX{j}(jj);
    idx = jj;
    parIdx = ISOparperm2.(strcat(isoVec{j}{:})){idx};
    parIdx = [parIdx,26:28];
    strarr = allParNames(parIdx);
    fprintf('%s%u%s\n','model',jj,'. ');
    fprintf('%s%f\n','Mean AIC: ',mean(ISOaic{j}(jj,:)));
    fprintf('%s%f\n','Relative likelihood: ',mean(ISOrelLikelihood{j}(jj)));
    for kk=1:length(strarr)
      fprintf('%s ',strarr{kk});
    end
    fprintf('\n');
    for counter = 1:length(parIdx) %iterate over estimated parameters
      replicateMedian = median(ISOparestStats{j}(parIdx(counter),jj,:),3);
      if parIdx(counter) <= 6 %fitness values
        fprintf('%s  ',num2str(1-replicateMedian,6)) %value conversion and output
        bestParsCutVals{j, parIdx(counter)} = [bestParsCutVals{j, parIdx(counter)}, 1-replicateMedian];
      elseif parIdx(counter) < 25 %resistance values
        fprintf('%s  ',num2str(1./(1-replicateMedian),6)) %value conversion and output
        bestParsCutVals{j, parIdx(counter)} = [bestParsCutVals{j, parIdx(counter)}, (1./(1-replicateMedian))];
      elseif parIdx(counter) >= 26 %basel. parameters
        if replicateMedian > 1
        end
        bestParsCutVals{j, parIdx(counter)} = [bestParsCutVals{j, parIdx(counter)}, replicateMedian];
        fprintf('%s  ',num2str(replicateMedian)) %value conversion and output
      end
    end
    fprintf('\n\n');
  end
end

maxParNumbers=zeros(1, nrIsolates);
for i=1:length(isoVec)
  maxParNumbers(i)=0;
  for j = 1:nrAllPars
    if ~isempty(bestParsCutVals{i,j})
      maxParNumbers(i)=maxParNumbers(i)+1;
    end
  end
end


%% ---------------------------
if SHOW_AIC_CURVE
  %plot residual curve
  figcnt=figcnt+1;
  figure(figcnt)
  isonames=cell(1,length(isoVec));
  
  colorVec={'b','r','k','g'};
  for isolate=1:length(isoVec)
    hold on
    legVec={};
    subplot(2,2,isolate)
    title('AIC of all parameter permutations')
    if size(ISOaic{isolate},2)<=1
      plot(ISOaic{isolate},colorVec{isolate});
    else
      boxplot([ISOaic{isolate}';ISOaic{isolate}'],'plotstyle','compact','colors',colorVec{isolate});
    end
    isonames{isolate}=char(isoVec{isolate}(1:end));
    ylabel('AIC');
    xlabel('model #/parameter permutation');  
    h = findobj(gca,'Type','line');
    set(h,'LineWidth',3);
    set(gca,'FontSize',10);
    set(gca,'LineWidth',3);
    set(get(gca,'title'),'Fontsize',18);
    set(get(gca,'xlabel'),'Fontsize',16);
    set(get(gca,'ylabel'),'Fontsize',16);
    axis tight
  end
  %legend(isonames);
end % IFSHOW_AIC_CURVE
%% ---------------------

%---------------------
if PRINT_TO_FILE
  %Print out medians and the percentiles for each of the best parameters
  %estimated to file
  fid = fopen(OUTFILENAME, 'w');
  fprintf(fid, '%s\n\n%s\n', 'Best parameter estimates.','Name Median 5%-prctl 95%-prctl');
  for i =1:length(isoVec)
    fprintf(fid, '%s\n', isoVec{i}{:});
    for j = 1:nrAllPars
      parVals = bestParsCutVals{i,j};
      if ~isempty(parVals)
        fprintf(fid, '%s :', allParNames{j});
        fprintf(fid,'%1.2f ',median(bestParsCutVals{i,j}));
        fprintf(fid,'%1.2f ',prctile(bestParsCutVals{i,j},5));
        fprintf(fid,'%1.2f ',prctile(bestParsCutVals{i,j},95));
        fprintf(fid,'\n');
      end
    end
    fprintf(fid,'\n');
  end
  fclose(fid);
end
%-----------------------

%-----------------------
if PAR_BOXPLOTS
  for isoIdx=1:nrIsolates
    count=1;
    MaxCounter1 = 0;
    MaxCounter2 = 0;
    MaxCounter3 = 0;
    subplotIndices=zeros(1,2);
    subplotIndices(1,1) = 2;
    subplotIndices(1,2)=ceil(maxParNumbers(isoIdx)/2);
    figcnt=figcnt+1;
    figure('Name',['Parameter estimates for isolate:',' ',char(strcat(isoVec{isoIdx}(1)))]);
    for k=1:nrAllPars
      if ~isnan(mean(ISOparestStats{isoIdx}(k,1:cutVec(isoIdx),:)))
        estimates =(squeeze(ISOparestStats{isoIdx}(k,1:cutVec(isoIdx),:)))';
        subplot(subplotIndices(1),subplotIndices(2),count);

        if k <= 6 %fitness values
          boxplot(1-estimates,'plotstyle','compact','medianstyle','line','outliersize',10,'symbol','.');
          %ylim([0,1.5]);
          title(strcat('f(',char(allParNames{k}),')'));
          if ~MaxCounter1
            ylabel('fitness');
            MaxCounter1 = 1;
          end
        elseif k < 25 %resistance values
          %boxplot(1./(1-estimates), 'outliersize',15,'symbol','.');
          boxplot(1./(1-estimates),'plotstyle','compact','medianstyle','line','outliersize',10,'symbol','.');
          set(gca,'YScale','log')
          title(strcat('FR(',char(allParNames{k}),')'));
          if ~MaxCounter2
            ylabel('fold resistance');
            MaxCounter2 = 1;
          end
        else
          %boxplot(estimates, 'outliersize',15,'symbol','.');
          boxplot(estimates,'plotstyle','compact','medianstyle','line','outliersize',10,'symbol','.');
          %ylim([0,1.5]);
          title(allParNames{k});
          if ~MaxCounter3
            ylabel('estimate');
            MaxCounter3 = 1;
          end
        end
        if count-1 > subplotIndices(1) * subplotIndices(2) - subplotIndices(2)
          xlabel('# Permutation');
        end
        %boxplot(reshape(ISOparestStats{isoIdx}(idx,1:cut,:),cut,nrReplicates)', 'outliersize',15,'symbol','.');
        set(gca,'XTickLabel',{' '});
        %xlabel('# Permutation');
        %ylabel('Estimate');
        %h_xlabel = get(gca,'XLabel');
        %j_xlabel = get(gca,'YLabel');
        
        h = findobj(gca,'Type','line');
        set(h,'LineWidth',3);
        
        set(gca,'FontSize',16);
        set(gca,'LineWidth',3);
        set(get(gca,'title'),'Fontsize',18);
        set(get(gca,'xlabel'),'Fontsize',16);
        set(get(gca,'ylabel'),'Fontsize',16);
        
        %fett(isoIdx);
        %set(h_xlabel,'FontSize',16);
        %set(j_xlabel,'FontSize',16);
        grid on;
        %ylim([0,1.5]);
        %title(mutNames{k},'FontSize', 18);
        count = count+1;
      end
    end
  end
end
%-----------------------
%%
%-----------------------
if PRED_CHECKS
  figcnt=figcnt+1;
  ColorVec={'b','y','m','g','c','r'};
  MarkerVec={'<','^','>','v','d'};
  Experiment ={'NVP';'NVP+3TC';'NVP+ADV';'NVP+3TC+ADV';'3TC+ADV';'no Drug'};
  for i=1:nrIsolates
    isolate = isoVec{i};
    nrReplicates=size(ISOparestStats{i},3);
    stdMat=zeros(nrReplicates*cutVec(i), nrExp.default);
    meanMat=zeros(nrReplicates*cutVec(i), nrExp.default);
    [NVP,Resistance,Fitness,BasicPars,passageData] = getparameters(isoVec{i});
    meanData = nanmean(passageData(:,2:end));
    stddevData = nanstd(passageData(:,2:end));
    for j = 1:cutVec(i)
      parIndex = [ISOparperm2.(strcat(isoVec{i}{:})){j}, BaseParidx(2:end)];
      for k = 1:nrReplicates
        k_est = ISOparestStats{i}(parIndex,j,k);
        [stats, meanResult, stdResult] = FitStoch(k_est, simultanExperiments);
        stdMat((j-1)*nrReplicates+k,:) = stdResult;
        meanMat((j-1)*nrReplicates+k,:) = meanResult;
      end
    end
    for q=1:nrExp.default
      figure(figcnt);
      errorbar(meanData(q), mean(meanMat(:,q)), std(meanMat(:,q)), MarkerVec{i},'Color','k','MarkerEdgeColor','k','MarkerFaceColor',ColorVec{q},'MarkerSize',30,'LineWidth', 6 )
      axis([10,50, 10, 50])
      hold on
      
      figure(figcnt+1);
      errorbar(stddevData(q), mean(stdMat(:,q)), std(stdMat(:,q)), MarkerVec{i},'Color','k','MarkerEdgeColor','k','MarkerFaceColor',ColorVec{q},'MarkerSize',30 ,'LineWidth',6)
      axis([0,47, 0, 47])
      hold on;
    end
  end
end

figure(figcnt)
xlabel({'Means of measured passage times, (days)'})
ylabel({'Means of estimated', 'passage times, (days)'})
for k=0:3:45
  line([k,k+2],[k,k+2],'Color',[1,0.6,0.5], 'LineStyle','-','LineWidth',8);
  hold on;
end

fs=45
set(gca, 'FontSize',fs)
set(gca,'LineWidth',8);
set(get(gca,'title'),'Fontsize',fs);
set(get(gca,'xlabel'),'Fontsize',fs);
set(get(gca,'ylabel'),'Fontsize',fs);
set(legend,'Fontsize',fs);

figure(figcnt+1)
xlabel({'Standard deviation of measured passage times, (days)'})
ylabel({'Standard deviation of estimated', 'passage times, (days)'})
for k=0:3:45
  line([k,k+2],[k,k+2],'Color',[1,0.6,0.5], 'LineStyle','-','LineWidth',8);
  hold on;
end

fs=45
set(gca, 'FontSize',fs)
set(gca,'LineWidth',8);
set(get(gca,'title'),'Fontsize',fs);
set(get(gca,'xlabel'),'Fontsize',fs);
set(get(gca,'ylabel'),'Fontsize',fs);
set(legend,'Fontsize',fs);

