function [stats, meanResult, stdResult] = FitStoch(k_est, simultanExperiments)
%
% Inference of resistance and fitness values using HIV passage experiment data.
%
% Computes the first passage time (FPT) of an HIV population given a
% starting number of viruses V0 and a maximal number Vmax. Furthermore it computes
% the residual error of the theoretical FPT with respect to experimentally observed
% passage times (see Barbara A. Rath, Kaveh Pouran Yousef, David K. Katzenstein,
% Robert W. Shafer, Christof Schütte, Max von Kleist, Thomas C. Merigan (2013)
% "HIV-1 Evolution in Response to Triple Reverse Transcriptase Inhibitor Induced Selective
% Pressure in vitro", PlosOne)
%
% Input: 1. 'k_est': vector of mutation and drug effect parameters to be estimated
%        2. 'simultanExperiments': a vector containing indices of experiments to be fitted
%                                simultaneously
%
% Returns: 1. 'stats': a vector of residual errors of means and standard deviations
%              of passage times between experimental data and theoretical model
%          2. 'meanResult': vector of modelled mean first passage times for each
%              drug application setting
%          3. 'stdResult': vector modelled standard deviations of passage times for each
%              drug application setting
%
% Order of using the code:
%  1. Adjust programm variables in getparameters.m (optional)
%  2. Run EstimateMutationalParameters.m
%  3. Run EstimationStats.m
%
% Max v. Kleist and Kaveh Yousef
%
% Copyright (C) 2013, Free University Berlin
% Contact: yoocef@mi.fu-berlin.de, vkleist@zedat.fu-berlin.de
%
% FOR ACADEMIC USE this program is free software; you can redistribute
% it and/or modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
% USA.

global allMutPars parIndex V0 Vmax NVP Resistance Fitness meanData nrExp nrPass
global stddevData SearchAlgorithm OnlyImportantMutations BasicPars isolate


%"hard-coded" values for parameters, can be adjusted
lenpars=length(allMutPars);
currpars =zeros(1,lenpars+4);
currpars (parIndex)=k_est;

%Assign 1) growth rate r_0, 2) mean of stochastic NRTI effect epsMean,
% 3) probability of NRTI effect, 4) IC50 value of NVP effect

%Check if growth rate is also estimated
%if ~isnan(k_est(end-3))
%  r_0 = k_est(end-3);
%else
%fix the growth rate parameter if pre-estimated separately
r_0=BasicPars.r_0;
%end

IC50= k_est(end);
prob1 = k_est(end-1);
epsMean1=k_est(end-2);

%Drug application settings to include for combined estimation
%nrExp =length(simultanExperiments);

%Output parameters
stats = zeros(nrExp.default*2+1,1);
meanResult= zeros(1, nrExp.default);
stdResult= zeros(1,nrExp.default);

%Auxiliary variables for first passage time computation
T1cumsum = cumsum(1./((Vmax-1:-1:V0))); %Term sum(1/k) in eq(5)
T2cumsum =  cumsum(T1cumsum./(Vmax-1:-1:V0));%From eq(6) last terms: sum(1/k)*sum(1/s)

%iterate over the drug application settings
for exprmnt = 1:nrExp.default
%%
  s=0;
  j=simultanExperiments(exprmnt); 
  meanvec=nan(1,nrPass.default*length(isolate));
  stdvec =nan(1,nrPass.default*length(isolate)); 
  passNr=0;
  for ii=1:length(isolate)
    iso=isolate{ii}; 
    if prod(double(iso=='Iso3')) && j>3
      continue;
    end
    %If drug application setting includes NRTIs
    %then include stochastic effects (due to low dose regime)
    if j==1 || j==6
      epsMean1=0;
      prob1=0;
    elseif j==2 || j==3 || j==4 || j==5
      prob1 = k_est(end-1);
      epsMean1=k_est(end-2);
    end;
    
    %Iterate over the HIV growth passages
    for i = 1:nrPass.default
     
      passNr=passNr+1;
      
      if OnlyImportantMutations
        lastidx = Resistance.important;
      else
        lastidx = length(Resistance.parIndex);
      end
            
      %Add resistance effects of mutations only if NVP is applied
      %(thus exclude experments 5 and 6)
      if j==5 || j==6
      else
        resistance = 1;
        for q = 1:length(Resistance.Names(1:lastidx))
          name = Resistance.Names{q};%deduce name of resistance mutation to be considered
          pidx = Resistance.parIndex(q);%
          %Corresponds to (prod__q FR(q)) in eq. (2) in the paper
          % Note that "1/(1 - currpars (pidx)*Resistance.(name)(j,i))"
          % corresponds to FR(q) in the paper. "Resistance.(name)(j,i)" is an
          % indicator function which is 1 if the mutation is present
          % otherwise 0, see getparameters.m
          % "currpars" is the actual parameter estimate for the resistance
          resistance = resistance*1/(1 - currpars (pidx)*Resistance.(name)(j,i+(ii-1)*nrPass.default));
        end
      end
      
      %Add fitness effects of mutations
      fitness=1;
      for r = 1:length(Fitness.Names)
        name = Fitness.Names{r};
        pidx = Fitness.parIndex(r);
        
        %Corresponds to Eq.(3) in the paper
        % Note that "(1 - currpars (pidx)*Fitness.(name)(j,i))"
        % corresponds to f(q) in the paper. "Fitness.(name)(j,i)" is an
        % indicator function which is 1 if the mutation is present
        % otherwise 0, see getparameters.m
        fitness = fitness*(1 - currpars (pidx)*Fitness.(name)(j,i+(ii-1)*nrPass.default));
      end
      % two dimensional vector [either NRTI takes effect, or not], because
      % NRTI action is assumed to be of stochastic nature
      oneMinusEpsilon = [(1-epsMean1), 1];
      if j < 5
        %Eq.(2) in the paper (Emax equation) of drug effect
        NVPeffect = IC50 * resistance / (IC50 * resistance + NVP(j,i+(ii-1)*nrPass.default));
      else
        NVPeffect = 1;
      end
      %Eq.(1) in the paper
      r_ij = r_0*NVPeffect* fitness.*oneMinusEpsilon;
      
      %First moments of first passage time (mean)
      %Eq.(4) in the paper
      % first entry defines the case where NRTI takes effect with
      % 'oneMinusEpsilon'; second one, if NRTI does not stake effect
      T1 = T1cumsum(end)./r_ij;
      
      %Raw second moments of first passage time (non-centralized variability)
      %Eq.(6) in the paper
      % first entry defines the case where NRTI takes effect with
      % 'oneMinusEpsilon'; second one, if NRTI does not stake effect
      T2 = 2*T2cumsum(end)./(r_ij.^2);
      
      %Standard deviations
      %Eq.(8) in the paper
      % two entries depending on NRTI effect or not
      stds   = [sqrt(T2(1) - T1(1)^2), sqrt(T2(2) - T1(2)^2)];
      
      %NRTI action is assumed to be of stochastic nature
      %epsProb denotes the probabilities of 'effect' or 'no effect'
      epsProb = [prob1, (1-prob1)];
      
      % Weighting of mean First passage time and stds with the probabilities of NRTI effect
      % (epsProb);
      meanvec(i) = sum(epsProb.*T1);%[related to the pooling eq. (9-10)]
      stdvec(i) = sqrt(sum(epsProb.*stds.^2) +   epsProb(2)*epsProb(1)*(T1(2)-T1(1))^2);%[related to the pooling eq. (9-10)]
      
      %auxiliary variable for eq.(10) [pairwise differences of mean FPT]
      if i > 1
        s = s + nansum((meanvec(i+(ii-1)*nrPass.default) - meanvec(1:i+(ii-1)*nrPass.default-1)).^2);
      end
    end
  end
  % Pool the means and variances over all passages of current experiment j
  % Corresponds to eq.(9-10) in the paper.
  meanEst = nanmean(meanvec);
  stdEst =  sqrt(nanmean(stdvec.^2) + s/(passNr^2));
  
  meanResult(exprmnt)=meanEst;%assign output for 'lsqcurvefit'
  stdResult(exprmnt)=stdEst;%assign output for 'lsqcurvefit'
  
  % Objective function weigthed by experimental data
  % this denotes the terms mu_pred/mu_exp and
  % sigma_pred/sigma_exp;
  % this also denotes the output variable if 'lsqcurvefit' is used (least-square; constrained
  % optimization)
  stats(exprmnt*2-1 : exprmnt*2,1) = [meanEst/meanData(j),stdEst/stddevData(j)]';
end
% simultaneous minimization of IC50 within the objective function
IC50idx=find(parIndex==28);
if length(IC50idx)>0
  stats(nrExp.default*2+1,1)=1+k_est(IC50idx);%penalizes very large IC50-value estimates
else
  stats(nrExp.default*2+1,1) = 1;
end

%--------------------------------------------------------------------------
% simultaneous minimization of FR(q) within the objective function
ResParIdces= parIndex(parIndex<25 & parIndex>=7); %original index
NrResParIdces = length(ResParIdces);
if ~isempty(ResParIdces)
    stats = [stats;(1/(2.*NrResParIdces)).*log10(1./(1-currpars(ResParIdces)'))]; 
end
% 0 - w*log10(FR); weight 'w' is chosen such that the contribution of all resistance 
% parameters to the obj. function is half compared to the contribution of
% the IC50 to the obj. function       
%--------------------------------------------------------------------------

%output variable if fminsearch used (unconstrained optimization)
if SearchAlgorithm == 2
     if ~isempty(ResParIdces)
        stats = sum((stats - [ones(2*nrExp.default+1, 1); zeros(NrResParIdces ,1)]).^2);%least squares 
     else
        stats = sum((stats - ones(2*nrExp.default+1, 1)).^2);%least squares  
    end
  %stats = sum((stats - ones(2*nrExp.default+1, 1)).^2);%least squares
end