function [NVP,Resistance,Fitness,BasicPars,passageData] = getparameters(isoParVec)

% getparameters.m
%
% Inference of resistance and fitness values using HIV passage experiment data.
%
% This function contains all important adjustable variables of the estimation program. 
% Furthermore it contains matrices indicating mutational patterns and drug
% application regimes during passage time experiments. (see Barbara A. Rath, 
% Kaveh Pouran Yousef, David K. Katzenstein, Robert W. Shafer, Christof Schütte, 
% Max von Kleist, Thomas C. Merigan (2013) "HIV-1 Evolution in Response to Triple Reverse 
% Transcriptase Inhibitor Induced Selective 
% Pressure in vitro", PlosOne)
%
% Input: 'Isolate': A string referring to the isolate strain number.
%                   The format should be 'Iso1', 'Iso2', etc.
% 
% Returns: 1. 'NVP': a NxM matrix containing drug doses of NVP within
%             N different drug application regimes and M different
%             passages.
%          2. 'Resistance': a MATLAB struct consisting of the following fields:
%              -'Resistance.Names': a cell array containing names of
%                resistance mutations w.r.t. baseline strain observed in 
%                the viral strain 'Isolate'.
%              -'Resistance.important': the (integer) number giving the size 
%                of the subset of important mutations which should be 
%                included into the estimation procedure. This subset may be the
%                result of previous studies which indicate significant resistance gain towards NVP.
%                At its maximum, this number it is the size of 'Resistance.Names',
%                indicating that all observed mutation parameters should be
%                included into estimation.
%              -'Resistance.AxxxB': an indicator table of size NxM, where the
%                entry (i,j) is 1 if the mutation of aminoacid A to aminoacid B
%                at position xxx of RT-sequence was selected at passage j or earlier   
%                within drug regime i, where j is in the range [1...M] and  
%                i is in the range [1...N]. N and M are as above (see parameter 'NVP').
%          3. 'Fitness':   a MATLAB struct consisting of the following fields:
%              - 'Fitness.Names': a cell array containing names of
%                observed mutations in viral strain 'Isolate',  which are known 
%                from previous studies to induce significant fitness
%                costs.
%              - 'Fitness.AxxxB': indicator table, same as 'Resistance.AxxxB' 
%                but describes fitness cost inducing mutations.   
%          4. 'BasicPars': a MATLAB struct containg basic strain specific 
%              parameters of viral growth, indepedent of mutational events.
%              So far, its only field is BasicPars.r_0, where r_0 is the 
%              viral growth rate of baseline strain (no mutations) in
%              absence of drug application.
%
% Order of using the code:
%  1. Adjust program variables in getparameters.m (optional)
%  2. Run EstimateMutationalParameters.m
%  3. Run EstimationStats.m
%
%
% Max v. Kleist and Kaveh Yousef, January 2013
%
% Copyright (C) 2013, Free University Berlin
% Contact: yoocef@mi.fu-berlin.de, vkleist@zedat.fu-berlin.de
%
% FOR ACADEMIC USE this program is free software; you can redistribute 
% it and/or modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
% USA.

%----------------------------------------------
%Index look-up table of all mutation parameters
%1. M184V  % 1-6: Fitness mutations
%2. N67S
%3. H208Y
%4. Y215C_D
%5. I35V
%6. W210L_K211R
%7. V106A  % 7-24: Resistance mutations
%8. Y188C
%9. V108I
%10. Y181C
%11. M184V
%12. D218E          
%13. K103N
%14. G190A
%15. E224K
%16. K101E
%17. F227L
%18. V106I
%19. Y108I
%20. V179I
%21. Y181C_G190A          
%22. V106M
%23. L228Q
%24. D218G

% following parameters are included per default 
%25. r_0 (basic growth rate) 
%26. epsMean1 (mean of stochastic NRTI effect)
%27. prob1 (probability of stochastic NRTI effect)
%28. IC50 value for dose-dependance of NVP drug effect
%----------------------------------------------


%specification , e.g. getparameters()
global allMutPars nrExp nrPass nrFitnessMuts

if ~nargin %System Initialization if 'getparameters' is called without isolate specification
    global isoVec V0 Vmax nrReplicates OnlyImportantMutations  
    global nrResistanceMuts SearchAlgorithm simultanExperiments MaxNumberParameters
    
    % Parameters referring to passage experiment settings
    %-----------------------------------------------
    %ADJUSTABLE PARAMETERS
    %-----------------------------------------------
    %Initial number of HI viruses.
    V0 = 1e2; 

    %Maximal number of HI viruses.
    Vmax=V0*100; 

    %Nr. estimation replicates for each parameter basis for estimation statistics
    %choose a value of at least 100 for a reliable parameter estimation. 
    nrReplicates = 49;

    %Names of isolates
    isoVec = {{'Iso1'}, {'Iso2', 'Iso3'}, {'Iso4'}, {'Iso5'}};
    
    %isoVec = {{'Iso1'}}
    %isoVec = {{'Iso2'}}
    
    %For constraining the number of estimated parameters, set 1, 
    %otherwise set 0.
    OnlyImportantMutations=0;

    %Vector of numbers of drug application regimes to be
    %estimated simultaneously
    simultanExperiments = [1,2,3,4,5,6];

    %Type of optimization algorithm:
    % 1: constrained optimization (suggested)
    % 2. unconstrained optimization
    SearchAlgorithm=1;
    %-----------------------------------------------

    %List of all mutation parameters to be included: first come mutations which
    %are known to induce fitness costs, then resistance mutations
    allMutPars= {'M184V', 'N67S', 'H208Y', 'Y215C_D', 'I35V', 'W210L_K211R', ...
              'V106A', 'Y188C', 'V108I' , 'Y181C',  'M184V',   'D218E', ...
              'K103N', 'G190A', 'E224K', 'K101E', 'F227L', ...
              'V106I', 'Y108I', 'V179I', 'Y181C_G190A', 'V106M', 'L228Q', 'D218G'};


    nrFitnessMuts=6;
    nrResistanceMuts=18;

    nrExp.default=6;
    nrPass.default=12;

    %[nrExp.default,nrPass.default]=size(NVP);

    %Resistance gain and fitness cost mutation tables.
    %Entry (i,j) is 1 if the corresponding mutation
    %has been selected within HIV population at
    %drug application setting i and passage j.

    Resistance=[];
    Fitness=[];
    BasicPars=[];
    passageData=[];
    MaxNumberParameters = [10;8];

elseif nargin > 0 %System variables, if 'getparameters' is called with isolate specification
    
    %Drug doses applied in each passage 1-12.
    %Passage 1      2       3       4       5       6       7       8       9       10      11      12
    NVP =   [0.01   0.02    0.04    0.08    0.16    0.32    0.64    1.28    2.56    5.12    10.24   20.48;%NVP only
             0.01   0.02    0.04    0.08    0.16    0.32    0.64    1.28    2.56    5.12    10.24   20.48;%NVP+3TC
             0.01   0.02    0.04    0.08    0.16    0.32    0.64    1.28    2.56    5.12    10.24   20.48;%NVP+ADV
             0.01   0.02    0.04    0.08    0.16    0.32    0.64    1.28    2.56    5.12    10.24   20.48;%NVP+3TC+ADV
             0      0       0       0       0       0       0       0       0       0       0       0;%3TC+ADV
             0      0       0       0       0       0       0       0       0       0       0       0];%no drug
    
    
    %Initialize structures
    Resistance=[];
    Fitness=[];
    BasicPars=[];
    passageData=[];
    nrExpVec=[];
    %---
    for ii =1:length(isoParVec)

        ResistancePrev = Resistance;
        FitnessPrev = Fitness;
        passageDataPrev = passageData;
        BasicParsPrev=  BasicPars;

        %load current data
        passageData = dlmread(strcat('./data/',isoParVec{ii},'_AllExperiments_indvPassageTimes.csv'),'\t');

        %if nr. of experiments is less than standard, fill the rows with zeros
        nrExpNew = size(passageData(:,2:end),2);         
        nrExpVec = [nrExpVec,nrExpNew];
        if nrExpNew < nrExp.default
          passageData = [passageData,NaN*zeros(nrPass.default, nrExp.default-nrExpNew)];
        end
        switch isoParVec{ii}
            case 'Iso1'                
            Resistance.Names = {'V106A','Y188C','V108I','Y181C'};
            Resistance.important = 4;
            Fitness.Names = {'M184V'}; 
            BasicPars.r_0=0.4196;%derived from pre-fitting experiments without drugs
                  %important resistance mutations
                  %Passage  1     2       3       4       5       6       7       8       9       10      11     12  
        Resistance.V106A = [0     0       0       0       0       0       0       0       0       0       0      1;  %NVP only
                            0     0       0       0       0       1       1       1       1       1       1      1;   %NVP+3TC
                            0     0       0       0       0       0       1       1       1       1       1      1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0]; %No drug

        Resistance.V108I = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       1       1       1       1       1       1       1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.Y181C = [0     0       0       0       0       0       1       1       1       1       1       1;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.Y188C = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       1       1       1       1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

                %Passage 1     2       3       4       5       6       7       8       9       10      11     12  
        Fitness.M184V = [1     1       1       1       1       1       1       1       1       1       1       0;  %NVP only
                         1     1       1       1       1       1       1       1       1       1       1       1;   %NVP+3TC
                         1     1       1       1       1       1       1       1       1       1       1       0; % NVP+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1]; %No drug

            case 'Iso2'
            Resistance.Names = {'V106A','V106I','Y181C','D218E','K103N','Y108I'}; 
            Resistance.important = 6 ;
            Fitness.Names = {'M184V','N67S'}; 
            BasicPars.r_0=0.39;%derived from pre-fitting experiments without drugs
                  %important resistance mutations
                  %Passage 1     2       3       4       5       6       7       8       9       10      11     12  
        Resistance.V106A = [0     0       0       0       0       0       1       1       1       1       1      1;  %NVP only
                            0     0       0       0       0       0       1       1       1       1       1      1;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0      0; % NVP+ADV
                            0     0       0       0       0       1       1       1       1       1       1      1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0]; %No drug

        Resistance.V106I = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       1       1       1       1       1       1       1       1       1       1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.Y181C = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       1       1       1; % NVP+ADV
                            0     0       0       0       0       0       0       1       1       1       1       1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.D218E = [0     1       1       1       1       1       1       1       1       1       1      1;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0      0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0      0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0]; %No drug

                         %less important mutations
        Resistance.K103N = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       1;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.Y108I = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       1       1       1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug    

                  %Fitness Mutations
                  %Passage 1     2       3       4       5       6       7       8       9       10      11     12                
        Fitness.M184V =   [1     1       1       1       1       1       1       0       0       0       0       0;  %NVP only
                           1     1       1       1       1       1       1       1       1       1       1       1;   %NVP+3TC
                           1     1       1       0       0       0       0       0       0       0       0       0; % NVP+ADV
                           1     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                           1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                           1     1       1       1       0       0       0       0       0       0       0       0]; %No drug

        Fitness.N67S =    [1     1       1       1       1       1       1       1       1       1       1       1;  %NVP only
                           1     1       1       1       1       1       0       0       0       0       0       0;   %NVP+3TC
                           1     1       1       1       1       1       1       1       1       1       1       1; % NVP+ADV
                           1     1       1       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                           1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                           1     1       1       1       1       1       1       1       1       1       1       1]; %No drug                  
            case 'Iso3'
            Resistance.Names = {'V106A','G190A','E224K','V106I','V179I','D218G','Y181C'};
            Resistance.important = 7 ;
            Fitness.Names = {'M184V','N67S'};    
            BasicPars.r_0=0.39;%derived from pre-fitting experiments without drugs
                  %important resistance mutations
                  %Passage  1     2       3       4       5       6       7       8       9       10      11     12  
        Resistance.V106A = [0     0       0       0       1       1       1       1       1       1       1      1;  %NVP only
                            0     0       0       0       0       1       1       1       1       1       1      1;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0      0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement 

        Resistance.V106I = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       1       1       1       1       1       1       1       1       1       1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement 

        Resistance.G190A = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       1       1       1       1       1; % NVP+AD
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement 

        Resistance.E224K = [0     0       0       0       0       0       0       0       0       0       1       1;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement 

                  %less important resistance mutations
                  %Passage  1     2       3       4       5       6       7       8       9       10      11     12  
        Resistance.V179I = [0     0       0       0       0       0       1       1       1       1       1      1;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0      0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0      0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement    

        Resistance.D218G = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement     


        Resistance.Y181C = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       1       1;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                            0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement                  


                %Passage 1     2       3       4       5       6       7       8       9       10      11     12  
        Fitness.M184V = [1     1       1       1       1       1       0       0       0       0       0       0;  %NVP only
                         1     1       1       1       1       1       1       1       1       1       1       1;   %NVP+3TC
                         1     1       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                         0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                         0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                         0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement

        Fitness.N67S =  [1     1       1       0       0       0       0       0       0       0       0       0; % NVP only
                         1     1       1       1       1       0       0       0       0       0       0       0; % NVP+3TC
                         1     1       1       1       1       1       1       1       1       1       1       1; % NVP+ADV                             
                         0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                         0     0       0       0       0       0       0       0       0       0       0       0; % No measurement
                         0     0       0       0       0       0       0       0       0       0       0       0]; % No measurement
            case 'Iso4'
            Resistance.Names = {'V106M','V108I','Y181C_G190A','L228Q','K101E','F227L'}; 
            Resistance.important = 4;
            Fitness.Names = {'M184V','H208Y','Y215C_D','I35V'} ;   
            BasicPars.r_0=0.33;%derived from pre-fitting experiments without drugs
                  %important resistance mutations
                  %Passage  1     2       3       4       5       6       7       8       9       10      11     12  
        Resistance.V106M = [0     0       0       0       0       0       0       0       0       0       0      0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0      0;   %NVP+3TC
                            0     0       0       0       1       1       1       1       1       1       1      1; % NVP+ADV
                            0     0       0       0       1       1       1       1       1       1       1      1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0]; %No drug

        Resistance.V108I = [0     0       1       1       1       1       1       1       1       1       1       1;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.Y181C_G190A = [0     0     0       0       0       0       0       0       0       0       0       0;  %NVP only
                                  0     0     0       0       0       1       1       1       1       1       1       1;   %NVP+3TC
                                  0     0     0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                                  0     0     0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                                  0     0     0       0       0       0       0       0       0       0       0        0; %3TC+ADV
                                  0     0       0     0       0       0       0       0       0       0       0        0]; %No drug

        Resistance.L228Q = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       1       1       1       1       1       1; % NVP+ADV
                            0     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug
        %less important resistance mutations               
        Resistance.K101E = [0     0       0       0       0       0       0       0       1       1       1       1;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.F227L = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       1       1       1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug    

                     %Fitness
               %Passage   1     2       3       4       5       6       7       8       9       10      11      12
         Fitness.M184V = [1     1       1       1       1       1       1       1       1       1       1       1;  %NVP only
                          1     1       1       1       1       1       1       1       1       1       1       1;   %NVP+3TC
                          1     1       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                          1     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                          1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                          1     1       0       0       0       0       0       0       0       0       0       0]; %No drug
                       %H208Y
         Fitness.H208Y = [1     0       0       0       0       0       0       0       0       0       0        0;  %NVP only
                          1     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                          1     1       1       1       1       1       1       1       1       1       1       1; % NVP+ADV
                          1     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                          1     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                          1     1       1       1       1       1       1       1       1       1       1       1]; %No drug

         Fitness.Y215C_D = [1     1       1       0       0       0       0       0       0       0       0       0;  %NVP only
                            1     1       1       1       1       1       1       1       1       1       1       1;   %NVP+3TC
                            1     1       1       1       1       1       1       1       1       1       1       1; % NVP+ADV
                            1     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                            1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                            1     1       1       1       1       1       1       1       1       0       0       0]; %No drug

        Fitness.I35V =  [1     1       1       1       1       1       1       1       0       0       0       0;  %NVP only
                         1     1       1       1       1       1       1       1       1       1       1       1;   %NVP+3TC
                         1     1       1       1       1       0       0       0       0       0       0       0; % NVP+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1]; %No drug                
            case 'Iso5'

            Resistance.Names = {'V106A','Y188C','V108I','Y181C','F227L'};
            Resistance.important = 4;
            Fitness.Names = {'M184V','W210L_K211R'} ;  
            BasicPars.r_0=0.36; %derived from pre-fitting experiments without drugs
            %important resistance mutations
                %Passage   1      2       3       4       5       6       7       8        9      10      11      12
        Resistance.V106A = [0     0       0       0       0       0       0       0       0       0       0      0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0      0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0      0; % NVP+ADV
                            0     0       0       0       0       0       0       1       1       1       1      1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0      0]; %No drug

        Resistance.V108I = [0     0       0       0       0       0       0       0       0       0       0      0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       1       1       1       1       1       1       1       1; % NVP+ADV
                            0     0       0       0       0       0       0       1       1       1       1       1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug

        Resistance.Y181C = [0     0       0       0       0       0       0       1       1       1       1       1;  %NVP only
                            0     0       0       0       0       0       1       1       1       1       1       1;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug   

        Resistance.Y188C = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       1       1       1; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug 

        %less important resistance mutations  
                 %Passage  1      2       3       4       5       6       7       8       9       10      11      12
        Resistance.F227L = [0     0       0       0       0       0       0       0       0       0       0       0;  %NVP only
                            0     0       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                            0     0       0       0       0       0       0       0       0       0       0       0; % NVP+ADV
                            0     0       0       0       0       0       0       0       0       0       0       1; % NVP+3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0; %3TC+ADV
                            0     0       0       0       0       0       0       0       0       0       0       0]; %No drug   

               %Passage 1      2       3       4       5       6       7       8       9       10      11      12
        Fitness.M184V = [1     1       1       0       0       0       0       0       0       0       0       0;  %NVP only
                         1     1       1       1       1       1       1       1       1       1       1       1;   %NVP+3TC
                         1     1       1       1       1       1       1       1       1       0       0       0; % NVP+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                         1     1       1       0       0       0       0       0       0       0       0       0]; %No drug

        Fitness.W210L_K211R = [0     0 0       0       0       0       0       0       0       0       0       0;  %NVP only
                         1     1       0       0       0       0       0       0       0       0       0       0;   %NVP+3TC
                         1     1       1       1       1       1       1       1       1       1       1       1; % NVP+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; % NVP+3TC+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1; %3TC+ADV
                         1     1       1       1       1       1       1       1       1       1       1       1]; %No drug                           
        end % switch 

        %Assign the corresponding indices to the mutations
        len=length(Fitness.Names);
        Fitness.parIndex=zeros(1,len);
        for i=1:len
          mutname=Fitness.Names{i};
          idx = strmatch(mutname,allMutPars(1:nrFitnessMuts),'exact'); 
          Fitness.parIndex(i)=idx; 
        end

        len=length(Resistance.Names);
        Resistance.parIndex=zeros(1,len);
        for i=1:len
          mutname=Resistance.Names{i};
          idx = strmatch(mutname,allMutPars(nrFitnessMuts+1:end),'exact');
          Resistance.parIndex(i)=idx+nrFitnessMuts; 
        end

        %Merge different isolates (if indicated in 'isoVec')
        if ii > 1      
          imptntprev=ResistancePrev.important;
          imprntcurr=Resistance.important;
          %ResistanceTmp.Names =  ResistancePrev.Names(1:imptntprev);
          %ResistanceTmp.parIndex =  ResistancePrev.parIndex(1:imptntprev);
          ResistanceTmp.Names={};
          ResistanceTmp.parIndex=[];
          FitnessTmp.Names={};
          FitnessTmp.parIndex=[];
          for j = 1:imprntcurr
            name=Resistance.Names{j};
            if length(find(Resistance.parIndex(j) == ResistancePrev.parIndex(1:imptntprev)))>0          
              ResistanceTmp.(name)=[ResistancePrev.(name),Resistance.(name)];
            else
              ResistanceTmp.(name)= [zeros(nrExp.default,nrPass.default*(ii-1)), Resistance.(name)];     
            end
            ResistanceTmp.parIndex = [ResistanceTmp.parIndex, Resistance.parIndex(j)];
            ResistanceTmp.Names = [ResistanceTmp.Names,name];
          end

          for j = 1:imptntprev
            if length(find(ResistancePrev.parIndex(j) == ResistanceTmp.parIndex))==0
              name=ResistancePrev.Names{j};
              ResistanceTmp.(name)= [ResistancePrev.(name), zeros(nrExp.default,nrPass.default)];
              ResistanceTmp.parIndex = [ResistanceTmp.parIndex, ResistancePrev.parIndex(j)];
              ResistanceTmp.Names = [ResistanceTmp.Names, name];
            end  
          end

          for j = 1:length(Fitness.parIndex)
            name=Fitness.Names{j};
            if length(find(Fitness.parIndex(j) == FitnessPrev.parIndex))>0
              FitnessTmp.(name)= [FitnessPrev.(name),Fitness.(name)];
            else    
              FitnessTmp.(name)= [zeros(nrExp.default,nrPass.default*(ii-1)),Fitness.(name)];
            end
              FitnessTmp.parIndex = [FitnessTmp.parIndex, Fitness.parIndex(j)];
              FitnessTmp.Names = [FitnessTmp.Names,name];
          end

          for j = 1:length(FitnessPrev.parIndex)
            if length(find(FitnessPrev.parIndex(j) == FitnessTmp.parIndex))==0
              disp('inside')
              name=FitnessPrev.Names{j};
              FitnessTmp.(name)= [FitnessPrev.(name), zeros(nrExp.default,nrPass.default)];
              FitnessTmp.parIndex = [FitnessTmp.parIndex, FitnessPrev.parIndex(j)];
              FitnessTmp.Names = [FitnessTmp.Names, name];
            end  
          end

          BasicPars.r_0 = [BasicPars.r_0,BasicParsPrev.r_0];
          
          passageData = [passageDataPrev;passageData];
          Resistance=ResistanceTmp;
          Fitness=FitnessTmp;
          Resistance.important = length(Resistance.parIndex);
        end
    end % for ii=1:length(isoParVec)

    BasicPars.r_0=mean(BasicPars.r_0);
    NVP=repmat(NVP,1,length(isoParVec));
end % if nargin 